@@ -20,7 +20,7 @@ class GerritAccountEmail(GerritBase):
         self.account = account
         self.gerrit = gerrit
         self.endpoint = f"/accounts/{self.account}/emails/{self.email}"
-        GerritBase.__init__(self)
+        super().__init__(self)
 
     def __str__(self):
         return self.email
